/*
 * Copyright (c) 2005, Joshua Wright <jwright@hasborg.com>
 *
 * $Id: bluecrypt.c,v 1.10 2005/09/07 22:50:17 jwright Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See COPYING for more
 * details.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
 
#include <stdint.h>
#include "bluecrypt.h"
#include "utils.h"
#include "tomcrypt.h"


/* E21 is used for generaring the link key LK_Ka */
int e21(uint8_t *in_rand, uint8_t *in_bdaddr, uint8_t *out_link_key)
{

	int i, err;
	uint8_t key[16], rand[16];
	symmetric_key skey;

	/* Copy the in_rand value for use as the key to A_r' and cyclic expand
	   BD_ADDR to use as the input data for A_r'. */
	for (i = 0; i < 16; i++) {
		key[i] = in_rand[i];
		rand[i] = ((uint8_t *) (in_bdaddr))[i % 6];
	}

	/* e21 uses LK_RAND as the key, with the last byte XOR'd by static 6 */
	key[15] ^= 6;

	/* Setup ksa */
	if ((err = saferp_setup(key, 16, 0, &skey)) != CRYPT_OK) {
		printf("Setup error: %s\n", error_to_string(err));
		return -1;
	}

	saferp_ecb_encrypt_aprime(rand, out_link_key, &skey);
	saferp_done(&skey);
	return 0;
}


/* E22 is used to produce K_init and K_master for Bluetooth.  K_init does not
   use E22 directly, rather is must be called after augmenting PIN */
int e22(uint8_t *in_key, uint8_t *in_rand, int pinlenprime, uint8_t *out_k) 
{
	int err;
	uint8_t rand[16];
	symmetric_key skey;

	/* Copy in_rand locally */
	memcpy(rand, in_rand, 16);

	/* XOR last byte of PIN' with L' */
	rand[15] ^= pinlenprime;

	/* Setup ksa */
	if ((err = saferp_setup(in_key, 16, 0, &skey)) != CRYPT_OK) {
		printf("Setup error: %s\n", error_to_string(err));
		return -1;
	}

	/* Encrypt */
	saferp_ecb_encrypt_aprime(rand, out_k, &skey);
	saferp_done(&skey);
	return 0;
}

/* E1 is used as the Bluetooth hashing function, generating the signed response
   (SRES) and authenticated ciphering offset (ACO).  The SRES value is used
   for mutual authentication during pairing. */
int e1(uint8_t *in_kab, uint8_t *in_rand, uint8_t *in_bdaddr,
	uint8_t *out_sres, uint8_t *out_aco)
{

	uint8_t kab_offset[16];
	uint8_t bdaddr_exp[16];
	uint8_t ar_result[16];
	uint8_t sresaco[16];
	int err, i;
	symmetric_key skey;

	/* Encrypt in_rand with K_ab as key using A_r */
	if ((err = saferp_setup(in_kab, 16, 0, &skey)) != CRYPT_OK) {
		printf("Setup error: %s\n", error_to_string(err));
		return -1;
	}
	saferp_ecb_encrypt(in_rand, ar_result, &skey);
	saferp_done(&skey);

	/* XOR ar_result with in_rand */
	for (i = 0; i < 16; i++) {
		ar_result[i] ^= in_rand[i];
	}

	/* Cyclic expand bd_addr to 16 bytes */
	for (i = 0; i < 16; i++) {
		bdaddr_exp[i] = in_bdaddr[i % 6];
	}

	/* Add bdaddr_exp with ar_result */
	for (i = 0; i < 16; i++) {
		ar_result[i] += bdaddr_exp[i];
	}

	/* Calculate K_ab offset (K_ab tilde) */
	kab_offset[0] = in_kab[0] + 233;
	kab_offset[1] = in_kab[1] ^ 229;
	kab_offset[2] = in_kab[2] + 223;
	kab_offset[3] = in_kab[3] ^ 193;
	kab_offset[4] = in_kab[4] + 179;
	kab_offset[5] = in_kab[5] ^ 167;
	kab_offset[6] = in_kab[6] + 149;
	kab_offset[7] = in_kab[7] ^ 131;
	kab_offset[8] = in_kab[8] ^ 233;
	kab_offset[9] = in_kab[9] + 229;
	kab_offset[10] = in_kab[10] ^ 223;
	kab_offset[11] = in_kab[11] + 193;
	kab_offset[12] = in_kab[12] ^ 179;
	kab_offset[13] = in_kab[13] + 167;
	kab_offset[14] = in_kab[14] ^ 149;
	kab_offset[15] = in_kab[15] + 131;

	/* Using K_ab offset as the key, encrypt ar_result using A_r' */
	if ((err = saferp_setup(kab_offset, 16, 0, &skey)) != CRYPT_OK) {
		printf("Setup error: %s\n", error_to_string(err));
		return -1;
	}
	saferp_ecb_encrypt_aprime(ar_result, sresaco, &skey);
	saferp_done(&skey);

	/* 16-byte response from A_r' contains 4 bytes SRES (MSB) and 12 bytes
	   ACO (LSB) */
	memcpy(out_sres, sresaco, 4);
	memcpy(out_aco, sresaco+4, 12);

	return 0;
}


int gen_kinit(uint8_t in_pinlen, uint8_t *in_pin, uint8_t *in_bdaddr,
                uint8_t *in_rand, uint8_t *out_kinit)
{

	/* 
	   e22 accepts the pin, bd_addr and a random number as
	   inputs, generating the K_init or K_master value.  To generate
	   K_init however, we have to modify in_rand with L' and concatenate
	   and expand PIN with the BD_ADDR.

	   L = PIN Length, in_pinlen
	   L' = min(16, L+sizeof(bd_addr)), L prime
	   PIN = in_pin, up to 16 bytes
	   PIN' = Modified pin, pinprime
	 */

	uint8_t pinprime[16];
	uint8_t pinlenprime;
	int i, j;

	/* PIN cannot be longer than 16 bytes */
	if (in_pinlen > 16)
		return -1;

	/* Combine PIN and BD_ADDR to produce PIN'.  This loop concatenates PIN
	   and BD_ADDR then does a cyclic expansion to fill 16 bytes */
	for (i = 0; i < 16;) {
		/* Copy the PIN into PIN' up to L */
		for (j = 0; (j < in_pinlen) && (i < 16); j++) {
			pinprime[i] = in_pin[j];
			i++;
		}
		/* Copy the 6 bytes of BD_ADDR into PIN' */
		for (j = 0; (j < 6) && (i < 16); j++) {
			pinprime[i] = in_bdaddr[j];
			i++;
		}
	}

	/* PIN' is L+sizeof(bdaddr) or 16, whichever is smaller */
	pinlenprime = (in_pinlen + 6 < 16) ? in_pinlen + 6 : 16;

	/* Call e22 with PIN', modified in_rand */
	if (e22(pinprime, in_rand, pinlenprime, out_kinit) != 0) {
		return -1;
	}

	return 0;
}


/* Wrapper to generate K_master */
int gen_kmaster(uint8_t in_key[16], uint8_t in_rand[16], uint8_t *out_k)
{
    /* The last byte of in_rand is XOR'd with 16 to generate K_Master */
	return(e22(in_key, in_rand, 16, out_k));
}


/* LK_COMB_RAND is observed transmission in LMP_COMB_KEY message */
int gen_lkrand(uint8_t *in_lkcombrand, uint8_t *in_kinit, uint8_t *out_lkrand) {
	int i;

	/* LK_RAND is LK_COMB_RAND XOR K_init */
	for (i=0; i < 16; i++) {
		out_lkrand[i] = in_lkcombrand[i] ^ in_kinit[i];
	}

	return 0;
}


/* K_ab is used as an input to E1 */
int gen_kab(uint8_t *in_bdaddr_a, uint8_t *in_bdaddr_b, uint8_t *in_lkrand_a,
	uint8_t *in_lkrand_b, uint8_t *out_kab) {

	uint8_t lk_ka[16], lk_kb[16];
	int i;

	/* K_ab is calculated with BD_ADDR and LK_RAND for each device.
	   LK_RAND is the product of XOR'ing LK_COMB_RAND with K_init.
	   LK_Ka and LK_Kb are the products of E21 with matching BD_ADDR
	   and LK_RAND inputs */

	if (e21(in_lkrand_a, in_bdaddr_a, lk_ka) != 0) {
		return -1;
	}

	if (e21(in_lkrand_b, in_bdaddr_b, lk_kb) != 0) {
		return -1;
	}

	for (i=0; i < 16; i++) {
		out_kab[i] = lk_ka[i] ^ lk_kb[i];
	}

	return 0;
}


/* SRES and ACO are used for mutual authentiation, encryption key usage */
int gen_sresaco(uint8_t *in_kab, uint8_t *in_aurand, uint8_t *in_bdaddr,
	uint8_t *out_sres, uint8_t *out_aco) {

	/* AU_RAND is the observed data in LMP_au_rand messages */
	return(e1(in_kab, in_aurand, in_bdaddr, out_sres, out_aco));
}


